@@ -29,31 +29,35 @@ pub struct ReleaseOpt {
     #[structopt(default_value)]
     pub level_or_version: crate::version::TargetVersion,
 
-    #[structopt(short = "m")]
     /// Semver metadata
+    #[structopt(short = "m")]
     pub metadata: Option<String>,
 
-    #[structopt(short = "c", long = "config")]
     /// Custom config file
+    #[structopt(short = "c", long = "config")]
     pub custom_config: Option<String>,
 
-    #[structopt(long)]
     /// Ignore implicit configuration files.
+    #[structopt(long)]
     pub isolated: bool,
 
     #[structopt(flatten)]
     pub config: ConfigArgs,
 
+    /// Token to use when uploading
+    #[structopt(long)]
+    pub token: Option<String>,
+
     /// Actually perform a release. Dry-run mode is the default
     #[structopt(short = "x", long)]
     pub execute: bool,
 
-    #[structopt(long)]
     /// Skip release confirmation and version preview
+    #[structopt(long)]
     pub no_confirm: bool,
 
-    #[structopt(long)]
     /// The name of tag for the previous release.
+    #[structopt(long)]
     pub prev_tag_name: Option<String>,
 
     #[structopt(flatten)]
@@ -68,106 +72,117 @@ impl ReleaseOpt {
 
 #[derive(Debug, StructOpt)]
 pub struct ConfigArgs {
-    #[structopt(long)]
     /// Sign both git commit and tag,
-    pub sign: bool,
+    #[structopt(long, overrides_with("no-sign"))]
+    sign: bool,
+    #[structopt(long, overrides_with("sign"), hidden(true))]
+    no_sign: bool,
 
-    #[structopt(long)]
     /// Sign git commit
-    pub sign_commit: bool,
+    #[structopt(long, overrides_with("no-sign-commit"))]
+    sign_commit: bool,
+    #[structopt(long, overrides_with("sign-commit"), hidden(true))]
+    no_sign_commit: bool,
 
-    #[structopt(long)]
     /// Sign git tag
-    pub sign_tag: bool,
+    #[structopt(long, overrides_with("no-sign-tag"))]
+    sign_tag: bool,
+    #[structopt(long, overrides_with("sign-tag"), hidden(true))]
+    no_sign_tag: bool,
 
-    #[structopt(long)]
     /// Git remote to push
-    pub push_remote: Option<String>,
-
     #[structopt(long)]
+    push_remote: Option<String>,
+
     /// Cargo registry to upload to
-    pub registry: Option<String>,
-
     #[structopt(long)]
+    registry: Option<String>,
+
+    #[structopt(long, overrides_with("no-publish"), hidden(true))]
+    publish: bool,
     /// Do not run cargo publish on release
-    pub skip_publish: bool,
+    #[structopt(long, overrides_with("publish"), visible_alias = "skip-publish")]
+    no_publish: bool,
 
-    #[structopt(long)]
+    #[structopt(long, overrides_with("no-push"), hidden(true))]
+    push: bool,
     /// Do not run git push in the last step
-    pub skip_push: bool,
+    #[structopt(long, overrides_with("push"), visible_alias = "skip-push")]
+    no_push: bool,
 
-    #[structopt(long)]
+    #[structopt(long, overrides_with("no-tag"), hidden(true))]
+    tag: bool,
     /// Do not create git tag
-    pub skip_tag: bool,
+    #[structopt(long, overrides_with("tag"), visible_alias = "skip-tag")]
+    no_tag: bool,
 
-    #[structopt(long)]
+    #[structopt(long, overrides_with("no-verify"), hidden(true))]
+    verify: bool,
     /// Don't verify the contents by building them
-    pub no_verify: bool,
+    #[structopt(long, overrides_with("verify"))]
+    no_verify: bool,
 
+    /// Specify how workspace dependencies on this crate should be handed.
     #[structopt(
         long,
         possible_values(&crate::config::DependentVersion::variants()),
         case_insensitive(true),
     )]
-    /// Specify how workspace dependencies on this crate should be handed.
-    pub dependent_version: Option<crate::config::DependentVersion>,
+    dependent_version: Option<crate::config::DependentVersion>,
 
-    #[structopt(long)]
     /// Prefix of git tag, note that this will override default prefix based on sub-directory
-    pub tag_prefix: Option<String>,
-
     #[structopt(long)]
+    tag_prefix: Option<String>,
+
     /// The name of the git tag.
-    pub tag_name: Option<String>,
-
     #[structopt(long)]
+    tag_name: Option<String>,
+
     /// Pre-release identifier(s) to append to the next development version after release
-    pub dev_version_ext: Option<String>,
-
     #[structopt(long)]
-    /// Do not create dev version after release
-    pub no_dev_version: bool,
+    dev_version_ext: Option<String>,
+
+    /// Create dev version after release
+    #[structopt(long, overrides_with("no-dev-version"))]
+    dev_version: bool,
+    #[structopt(long, overrides_with("dev-version"), hidden(true))]
+    no_dev_version: bool,
 
-    #[structopt(long)]
     /// Provide a set of features that need to be enabled
-    pub features: Vec<String>,
-
     #[structopt(long)]
+    features: Vec<String>,
+
     /// Enable all features via `all-features`. Overrides `features`
-    pub all_features: bool,
-
     #[structopt(long)]
-    /// Token to use when uploading
-    pub token: Option<String>,
+    all_features: bool,
 }
 
 impl ConfigArgs {
     pub fn to_config(&self) -> crate::config::Config {
         crate::config::Config {
-            sign_commit: self
-                .sign
-                .then(|| true)
-                .or_else(|| self.sign_commit.then(|| true)),
-            sign_tag: self
-                .sign
-                .then(|| true)
-                .or_else(|| self.sign_tag.then(|| true)),
+            sign_commit: resolve_bool_arg(self.sign_commit, self.no_sign_commit)
+                .or_else(|| self.sign()),
+            sign_tag: resolve_bool_arg(self.sign_tag, self.no_sign_tag).or_else(|| self.sign()),
             push_remote: self.push_remote.clone(),
             registry: self.registry.clone(),
-            disable_publish: self.skip_publish.then(|| true),
-            no_verify: self.no_verify.then(|| true),
-            disable_push: self.skip_push.then(|| true),
+            publish: resolve_bool_arg(self.publish, self.no_publish),
+            verify: resolve_bool_arg(self.verify, self.no_verify),
+            push: resolve_bool_arg(self.push, self.no_push),
             dev_version_ext: self.dev_version_ext.clone(),
-            no_dev_version: self.no_dev_version.then(|| true),
+            dev_version: resolve_bool_arg(self.dev_version, self.no_dev_version),
             tag_prefix: self.tag_prefix.clone(),
             tag_name: self.tag_name.clone(),
-            disable_tag: self.skip_tag.then(|| true),
+            tag: resolve_bool_arg(self.tag, self.no_tag),
             enable_features: (!self.features.is_empty()).then(|| self.features.clone()),
             enable_all_features: self.all_features.then(|| true),
             dependent_version: self.dependent_version,
             ..Default::default()
         }
     }
+
+    fn sign(&self) -> Option<bool> {
+        resolve_bool_arg(self.sign, self.no_sign)
+    }
 }
 
 #[derive(StructOpt, Debug, Clone)]
@@ -198,3 +213,12 @@ impl Verbosity {
         }
     }
 }
+
+fn resolve_bool_arg(yes: bool, no: bool) -> Option<bool> {
+    match (yes, no) {
+        (true, false) => Some(true),
+        (false, true) => Some(false),
+        (false, false) => None,
+        (_, _) => unreachable!("StructOpt should make this impossible"),
+    }
+}
