@@ -152,7 +152,7 @@ fn release_packages<'m>(
                             prev_tag_name,
                             crate_name
                         );
-                    } else if !pkg.config.no_dev_version() {
+                    } else if pkg.config.dev_version() {
                         log::debug!("Ignoring lock file change since {}; could be a pre-release version bump.", prev_tag_name);
                     } else {
                         lock_changed = true;
@@ -175,7 +175,7 @@ fn release_packages<'m>(
                     );
                     changed_pkgs.insert(&pkg.meta.id);
                     changed_pkgs.extend(pkg.dependents.iter().map(|d| &d.pkg.id));
-                } else if lock_changed && pkg.bin && !pkg.config.no_dev_version() {
+                } else if lock_changed && pkg.bin && pkg.config.dev_version() {
                     log::debug!(
                         "Lock file changed for {} since {}, assuming its relevant",
                         crate_name,
@@ -188,7 +188,7 @@ fn release_packages<'m>(
                     log::warn!(
                         "Updating {} to {} despite no changes made since tag {}",
                         crate_name,
-                        version.version_string,
+                        version.full_version_string,
                         prev_tag_name
                     );
                 }
@@ -226,20 +226,46 @@ fn release_packages<'m>(
         log::warn!("{} is behind {}/{}", branch, git_remote, branch);
     }
 
+    let mut is_shared = true;
+    let mut shared_version: Option<version::Version> = None;
+    for pkg in pkgs {
+        if let Some(version) = pkg.version.as_ref() {
+            if pkg.config.shared_version() && pkg.version.is_some() {
+                if let Some(shared_version) = shared_version.as_ref() {
+                    if shared_version.bare_version != version.bare_version {
+                        is_shared = false;
+                        log::error!(
+                            "{} has version {}, should be {}",
+                            pkg.meta.name,
+                            version.bare_version,
+                            shared_version.bare_version_string
+                        );
+                    }
+                } else {
+                    shared_version = Some(version.clone());
+                }
+            }
+        }
+    }
+    if !is_shared {
+        log::error!("Crate versions deviated, aborting");
+        return Ok(110);
+    }
+
     // STEP 1: Release Confirmation
     if !dry_run && !args.no_confirm {
         let prompt = if pkgs.len() == 1 {
             let pkg = pkgs[0];
             let crate_name = pkg.meta.name.as_str();
             let base = pkg.version.as_ref().unwrap_or(&pkg.prev_version);
-            format!("Release {} {}?", crate_name, base.version_string)
+            format!("Release {} {}?", crate_name, base.full_version_string)
         } else {
             let mut buffer: Vec<u8> = vec![];
             writeln!(&mut buffer, "Release").unwrap();
             for pkg in pkgs {
                 let crate_name = pkg.meta.name.as_str();
                 let base = pkg.version.as_ref().unwrap_or(&pkg.prev_version);
-                writeln!(&mut buffer, "  {} {}", crate_name, base.version_string).unwrap();
+                writeln!(&mut buffer, "  {} {}", crate_name, base.full_version_string).unwrap();
             }
             write!(&mut buffer, "?").unwrap();
             String::from_utf8(buffer).expect("Only valid UTF-8 has been written")
@@ -258,7 +284,8 @@ fn release_packages<'m>(
         let crate_name = pkg.meta.name.as_str();
 
         if let Some(version) = pkg.version.as_ref() {
-            let new_version_string = version.version_string.as_str();
+            let prev_version_string = pkg.prev_version.full_version_string.as_str();
+            let new_version_string = version.full_version_string.as_str();
             log::info!("Update {} to version {}", crate_name, new_version_string);
             if !dry_run {
                 cargo::set_package_version(pkg.manifest_path, new_version_string)?;
@@ -273,14 +300,14 @@ fn release_packages<'m>(
             if !pkg.config.pre_release_replacements().is_empty() {
                 // try replacing text in configured files
                 let template = Template {
-                    prev_version: Some(&pkg.prev_version.version_string),
+                    prev_version: Some(&prev_version_string),
                     version: Some(new_version_string),
                     crate_name: Some(crate_name),
                     date: Some(NOW.as_str()),
                     tag_name: pkg.tag.as_deref(),
                     ..Default::default()
                 };
-                let prerelease = !version.version.pre.is_empty();
+                let prerelease = version.is_prerelease();
                 do_file_replacements(
                     pkg.config.pre_release_replacements(),
                     &template,
@@ -295,7 +322,7 @@ fn release_packages<'m>(
                 let pre_rel_hook = pre_rel_hook.args();
                 log::debug!("Calling pre-release hook: {:?}", pre_rel_hook);
                 let envs = maplit::btreemap! {
-                    OsStr::new("PREV_VERSION") => pkg.prev_version.version_string.as_ref(),
+                    OsStr::new("PREV_VERSION") => prev_version_string.as_ref(),
                     OsStr::new("NEW_VERSION") => new_version_string.as_ref(),
                     OsStr::new("DRY_RUN") => OsStr::new(if dry_run { "true" } else { "false" }),
                     OsStr::new("CRATE_NAME") => OsStr::new(crate_name),
@@ -313,11 +340,11 @@ fn release_packages<'m>(
                 }
             }
 
-            if ws_config.consolidate_commits() {
+            if pkg.config.consolidate_commits() {
                 shared_commit = true;
             } else {
                 let template = Template {
-                    prev_version: Some(&pkg.prev_version.version_string),
+                    prev_version: Some(prev_version_string),
                     version: Some(new_version_string),
                     crate_name: Some(crate_name),
                     date: Some(NOW.as_str()),
@@ -335,6 +362,9 @@ fn release_packages<'m>(
     if shared_commit {
         let shared_commit_msg = {
             let template = Template {
+                version: shared_version
+                    .as_ref()
+                    .map(|v| v.full_version_string.as_str()),
                 date: Some(NOW.as_str()),
                 ..Default::default()
             };
@@ -353,7 +383,7 @@ fn release_packages<'m>(
 
     // STEP 3: cargo publish
     for pkg in pkgs {
-        if !pkg.config.disable_publish() {
+        if pkg.config.publish() {
             let crate_name = pkg.meta.name.as_str();
             let base = pkg.version.as_ref().unwrap_or(&pkg.prev_version);
 
@@ -362,30 +392,32 @@ fn release_packages<'m>(
             let features = &pkg.features;
             if !cargo::publish(
                 dry_run,
-                pkg.config.no_verify(),
+                pkg.config.verify(),
                 pkg.manifest_path,
                 features,
                 pkg.config.registry(),
-                args.config.token.as_ref().map(AsRef::as_ref),
+                args.token.as_ref().map(AsRef::as_ref),
             )? {
                 return Ok(103);
             }
             let timeout = std::time::Duration::from_secs(300);
 
             if pkg.config.registry().is_none() {
-                cargo::wait_for_publish(crate_name, &base.version_string, timeout, dry_run)?;
+                cargo::wait_for_publish(crate_name, &base.full_version_string, timeout, dry_run)?;
                 // HACK: Even once the index is updated, there seems to be another step before the publish is fully ready.
                 // We don't have a way yet to check for that, so waiting for now in hopes everything is ready
                 if !dry_run {
                     let publish_grace_sleep = std::env::var("PUBLISH_GRACE_SLEEP")
-                        .unwrap_or_else(|_| String::from("5"))
+                        .unwrap_or_else(|_| Default::default())
                         .parse()
-                        .unwrap_or(5);
-                    log::info!(
-                        "Waiting an additional {} seconds for crates.io to update its indices...",
-                        publish_grace_sleep
-                    );
-                    std::thread::sleep(std::time::Duration::from_secs(publish_grace_sleep));
+                        .unwrap_or(0);
+                    if 0 < publish_grace_sleep {
+                        log::info!(
+                            "Waiting an additional {} seconds for crates.io to update its indices...",
+                            publish_grace_sleep
+                        );
+                        std::thread::sleep(std::time::Duration::from_secs(publish_grace_sleep));
+                    }
                 }
             } else {
                 log::debug!("Not waiting for publish because the registry is not crates.io and doesn't get updated automatically");
@@ -408,8 +440,8 @@ fn release_packages<'m>(
 
             let base = pkg.version.as_ref().unwrap_or(&pkg.prev_version);
             let template = Template {
-                prev_version: Some(&pkg.prev_version.version_string),
-                version: Some(&base.version_string),
+                prev_version: Some(&pkg.prev_version.full_version_string),
+                version: Some(&base.full_version_string),
                 crate_name: Some(crate_name),
                 tag_name: Some(tag_name),
                 date: Some(NOW.as_str()),
@@ -427,12 +459,13 @@ fn release_packages<'m>(
 
     // STEP 6: bump version
     let mut shared_commit = false;
+    let mut shared_post_version: Option<version::Version> = None;
     for pkg in pkgs {
         if let Some(version) = pkg.post_version.as_ref() {
             let cwd = pkg.package_path;
             let crate_name = pkg.meta.name.as_str();
 
-            let updated_version_string = version.version_string.as_ref();
+            let updated_version_string = version.full_version_string.as_ref();
             log::info!(
                 "Starting {}'s next development iteration {}",
                 crate_name,
@@ -445,8 +478,8 @@ fn release_packages<'m>(
             }
             let base = pkg.version.as_ref().unwrap_or(&pkg.prev_version);
             let template = Template {
-                prev_version: Some(&pkg.prev_version.version_string),
-                version: Some(&base.version_string),
+                prev_version: Some(&pkg.prev_version.full_version_string),
+                version: Some(&base.full_version_string),
                 crate_name: Some(crate_name),
                 date: Some(NOW.as_str()),
                 tag_name: pkg.tag.as_deref(),
@@ -463,12 +496,16 @@ fn release_packages<'m>(
                     dry_run,
                 )?;
             }
-            let commit_msg = template.render(pkg.config.post_release_commit_message());
 
-            if ws_config.consolidate_commits() {
+            if pkg.config.shared_version() && shared_post_version.is_none() {
+                shared_post_version = Some(version.clone());
+            }
+            if pkg.config.consolidate_commits() {
                 shared_commit = true;
             } else {
                 let sign = pkg.config.sign_commit();
+
+                let commit_msg = template.render(pkg.config.post_release_commit_message());
                 if !git::commit_all(cwd, &commit_msg, sign, dry_run)? {
                     return Ok(105);
                 }
@@ -478,7 +515,13 @@ fn release_packages<'m>(
     if shared_commit {
         let shared_commit_msg = {
             let template = Template {
+                version: shared_version
+                    .as_ref()
+                    .map(|v| v.full_version_string.as_str()),
                 date: Some(NOW.as_str()),
+                next_version: shared_post_version
+                    .as_ref()
+                    .map(|v| v.full_version_string.as_str()),
                 ..Default::default()
             };
             template.render(ws_config.post_release_commit_message())
@@ -495,11 +538,10 @@ fn release_packages<'m>(
     }
 
     // STEP 7: git push
-    if !ws_config.disable_push() {
-        let shared_push = ws_config.consolidate_pushes();
-
+    if ws_config.push() {
+        let mut shared_push = false;
         for pkg in pkgs {
-            if pkg.config.disable_push() {
+            if !pkg.config.push() {
                 continue;
             }
 
@@ -511,7 +553,9 @@ fn release_packages<'m>(
                 }
             }
 
-            if !shared_push {
+            if pkg.config.consolidate_pushes() {
+                shared_push = true
+            } else if !shared_push {
                 log::info!("Pushing HEAD to {}", git_remote);
                 if !git::push(
                     cwd,
@@ -524,7 +568,6 @@ fn release_packages<'m>(
                 }
             }
         }
-
         if shared_push {
             log::info!("Pushing HEAD to {}", git_remote);
             if !git::push(
@@ -638,17 +681,14 @@ impl<'m> PackageRelease<'m> {
 
             release_config
         };
-        if config.disable_release() {
+        if !config.release() {
             log::debug!("Disabled in config, skipping {}", manifest_path.display());
             return Ok(None);
         }
 
         let is_root = git_root == cwd;
 
-        let prev_version = version::Version {
-            version: pkg_meta.version.clone(),
-            version_string: pkg_meta.version.to_string(),
-        };
+        let prev_version = version::Version::from(pkg_meta.version.clone());
 
         let package_content = cargo::package_content(manifest_path)?;
 
@@ -658,8 +698,8 @@ impl<'m> PackageRelease<'m> {
             prev_tag.to_owned()
         } else {
             let mut template = Template {
-                prev_version: Some(&prev_version.version_string),
-                version: Some(&prev_version.version_string),
+                prev_version: Some(&prev_version.full_version_string),
+                version: Some(&prev_version.full_version_string),
                 crate_name: Some(pkg_meta.name.as_str()),
                 ..Default::default()
             };
@@ -672,7 +712,7 @@ impl<'m> PackageRelease<'m> {
 
         let version = args
             .level_or_version
-            .bump(&prev_version.version, args.metadata.as_deref())?;
+            .bump(&prev_version.full_version, args.metadata.as_deref())?;
         let is_pre_release = version
             .as_ref()
             .map(version::Version::is_prerelease)
@@ -683,12 +723,10 @@ impl<'m> PackageRelease<'m> {
 
         let base = version.as_ref().unwrap_or(&prev_version);
 
-        let tag = if config.disable_tag() {
-            None
-        } else {
+        let tag = if config.tag() {
             let mut template = Template {
-                prev_version: Some(&prev_version.version_string),
-                version: Some(&base.version_string),
+                prev_version: Some(&prev_version.full_version_string),
+                version: Some(&base.full_version_string),
                 crate_name: Some(pkg_meta.name.as_str()),
                 ..Default::default()
             };
@@ -697,18 +735,16 @@ impl<'m> PackageRelease<'m> {
             let tag_prefix = template.render(tag_prefix);
             template.prefix = Some(&tag_prefix);
             Some(template.render(config.tag_name()))
+        } else {
+            None
         };
 
-        let post_version = if !is_pre_release && !config.no_dev_version() {
-            let mut post = base.version.clone();
+        let post_version = if !is_pre_release && config.dev_version() {
+            let mut post = base.full_version.clone();
             post.increment_patch();
             post.pre = semver::Prerelease::new(config.dev_version_ext())?;
-            let post_string = post.to_string();
 
-            Some(version::Version {
-                version: post,
-                version_string: post_string,
-            })
+            Some(version::Version::from(post))
         } else {
             None
         };
@@ -754,13 +790,13 @@ fn update_dependent_versions(
     version: &version::Version,
     dry_run: bool,
 ) -> Result<(), error::FatalError> {
-    let new_version_string = version.version_string.as_str();
+    let new_version_string = version.bare_version_string.as_str();
     let mut dependents_failed = false;
     for dep in pkg.dependents.iter() {
         match pkg.config.dependent_version() {
             config::DependentVersion::Ignore => (),
             config::DependentVersion::Warn => {
-                if !dep.req.matches(&version.version) {
+                if !dep.req.matches(&version.bare_version) {
                     log::warn!(
                         "{}'s dependency on {} `{}` is incompatible with {}",
                         dep.pkg.name,
@@ -771,7 +807,7 @@ fn update_dependent_versions(
                 }
             }
             config::DependentVersion::Error => {
-                if !dep.req.matches(&version.version) {
+                if !dep.req.matches(&version.bare_version) {
                     log::warn!(
                         "{}'s dependency on {} `{}` is incompatible with {}",
                         dep.pkg.name,
@@ -783,8 +819,8 @@ fn update_dependent_versions(
                 }
             }
             config::DependentVersion::Fix => {
-                if !dep.req.matches(&version.version) {
-                    let new_req = version::set_requirement(dep.req, &version.version)?;
+                if !dep.req.matches(&version.bare_version) {
+                    let new_req = version::set_requirement(dep.req, &version.bare_version)?;
                     if let Some(new_req) = new_req {
                         log::info!(
                             "Fixing {}'s dependency on {} to `{}` (from `{}`)",
@@ -804,7 +840,7 @@ fn update_dependent_versions(
                 }
             }
             config::DependentVersion::Upgrade => {
-                let new_req = version::set_requirement(dep.req, &version.version)?;
+                let new_req = version::set_requirement(dep.req, &version.bare_version)?;
                 if let Some(new_req) = new_req {
                     log::info!(
                         "Upgrading {}'s dependency on {} to `{}` (from `{}`)",
